### Name: setx
### Title: Setting Explanatory Variable Values
### Aliases: setx
### Keywords: file

### ** Examples

# Unconditional prediction:
data(turnout)
z.out <- zelig(vote ~ race + educate, model = "logit", data = turnout)
x.out <- setx(z.out)
s.out <- sim(z.out, x = x.out)

# Unconditional prediction with all observations:
x.out <- setx(z.out, fn = NULL)
s.out <- sim(z.out, x = x.out)

# Unconditional prediction with out of sample data:
z.out <- zelig(vote ~ race + educate, model = "logit",
               data = turnout[1:1000,])
x.out <- setx(z.out, data = turnout[1001:2000,])
s.out <- sim(z.out, x = x.out)

# Using a user-defined function in fn:
## Not run: 
##D quants <- function(x)
##D   quantile(x, 0.25)
##D x.out <- setx(z.out, fn = list(numeric = quants))
## End(Not run)

# Conditional prediction:  
## Not run: 
##D library(MatchIt)
##D data(lalonde)
##D match.out <- matchit(treat ~ age + educ + black + hispan + married + 
##D                      nodegree + re74 + re75, data = lalonde)
##D z.out <- zelig(re78 ~ distance, data = match.data(match.out, "control"), 
##D                model = "ls")
##D x.out <- setx(z.out, fn = NULL, data = match.data(match.out, "treat"),
##D               cond = TRUE)
##D s.out <- sim(z.out, x = x.out)
## End(Not run)



